"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigAction = void 0;
const code_analyzer_core_1 = require("@salesforce/code-analyzer-core");
const LogWriter_1 = require("../writers/LogWriter");
const WorkspaceUtil_1 = require("../utils/WorkspaceUtil");
const LogEventListener_1 = require("../listeners/LogEventListener");
class ConfigAction {
    dependencies;
    constructor(dependencies) {
        this.dependencies = dependencies;
    }
    async execute(input) {
        // We need the user's Config and the default Config.
        const userConfig = this.dependencies.configFactory.create(input['config-file']);
        const defaultConfig = code_analyzer_core_1.CodeAnalyzerConfig.withDefaults();
        // We always add a Logger Listener to the appropriate listeners list, because we should always be logging.
        const logEventLogger = new LogEventListener_1.LogEventLogger(await LogWriter_1.LogFileWriter.fromConfig(userConfig));
        this.dependencies.logEventListeners.push(logEventLogger);
        // The User's config produces one Core.
        const userCore = new code_analyzer_core_1.CodeAnalyzer(userConfig);
        // The Default config produces two Cores, since we have to run two selections.
        const defaultCoreForAllRules = new code_analyzer_core_1.CodeAnalyzer(defaultConfig);
        const defaultCoreForSelectRules = new code_analyzer_core_1.CodeAnalyzer(defaultConfig);
        // LogEventListeners should start listening as soon as the User Core is instantiated, since it can start emitting
        // relevant events basically immediately.
        this.dependencies.logEventListeners.forEach(listener => listener.listen(userCore));
        // Only the File Logger should listen to the Default Cores, since we don't want to bother the user with redundant
        // logs printed to the console.
        logEventLogger.listen(defaultCoreForAllRules);
        logEventLogger.listen(defaultCoreForSelectRules);
        const enginePlugins = this.dependencies.pluginsFactory.create();
        const enginePluginModules = userConfig.getCustomEnginePluginModules();
        const addEnginePromises = [
            // Assumption: It's safe for the different cores to share the same plugins, because all currently existent
            // plugins return unique engines every time. This code may fail or behave unexpectedly if a plugin reuses engines.
            ...enginePlugins.map(enginePlugin => userCore.addEnginePlugin(enginePlugin)),
            ...enginePlugins.map(enginePlugin => defaultCoreForAllRules.addEnginePlugin(enginePlugin)),
            ...enginePlugins.map(enginePlugin => defaultCoreForSelectRules.addEnginePlugin(enginePlugin)),
            ...enginePluginModules.map(pluginModule => userCore.dynamicallyAddEnginePlugin(pluginModule)),
            // Assumption: Every engine's default configuration is sufficient to allow that engine to be instantiated,
            // or throw a clear error indicating the problem.
            ...enginePluginModules.map(pluginModule => defaultCoreForAllRules.dynamicallyAddEnginePlugin(pluginModule)),
            ...enginePluginModules.map(pluginModule => defaultCoreForSelectRules.dynamicallyAddEnginePlugin(pluginModule)),
        ];
        await Promise.all(addEnginePromises);
        const userSelectOptions = input.workspace
            ? { workspace: await (0, WorkspaceUtil_1.createWorkspace)(userCore, input.workspace) }
            : undefined;
        const defaultSelectOptionsForAllRules = input.workspace
            ? { workspace: await (0, WorkspaceUtil_1.createWorkspace)(defaultCoreForAllRules, input.workspace) }
            : undefined;
        const defaultSelectOptionsForSelectRules = input.workspace
            ? { workspace: await (0, WorkspaceUtil_1.createWorkspace)(defaultCoreForSelectRules, input.workspace) }
            : undefined;
        // EngineProgressListeners should start listening right before we call the Cores' `.selectRules()` methods, since
        // that's when progress events can start being emitted.
        this.dependencies.progressEventListeners.forEach(listener => listener.listen(userCore, defaultCoreForAllRules, defaultCoreForSelectRules));
        const [userRules, allDefaultRules, selectedDefaultRules] = await Promise.all([
            // The user Core should respect the user's selection criteria.
            userCore.selectRules(input['rule-selector'], userSelectOptions),
            // One of the Default Cores should always use "all", to minimize the likelihood of a rule's default state
            // being unavailable for comparison.
            defaultCoreForAllRules.selectRules(['all'], defaultSelectOptionsForAllRules),
            // One of the Default Cores should use the same selectors as the User Core, so we know what rules are available
            // by default.
            defaultCoreForSelectRules.selectRules(input['rule-selector'], defaultSelectOptionsForSelectRules)
        ]);
        // After the Cores are done running, the listeners need to be told to stop, since some of them have persistent UI
        // elements or file handlers that must be gracefully ended.
        this.dependencies.progressEventListeners.forEach(listener => listener.stopListening());
        this.dependencies.logEventListeners.forEach(listener => listener.stopListening());
        // We need the Set of all Engines that returned rules for the user's selection on both the Default and User Cores.
        const relevantEngines = new Set([...userRules.getEngineNames(), ...selectedDefaultRules.getEngineNames()]);
        const userConfigContext = {
            config: userConfig,
            core: userCore,
            rules: userRules
        };
        const defaultConfigContext = {
            config: defaultConfig,
            core: defaultCoreForAllRules,
            rules: allDefaultRules
        };
        const configModel = this.dependencies.modelGenerator(relevantEngines, userConfigContext, defaultConfigContext);
        this.dependencies.viewer.view(configModel);
        this.dependencies.writer?.write(configModel);
        return Promise.resolve();
    }
    static createAction(dependencies) {
        return new ConfigAction(dependencies);
    }
}
exports.ConfigAction = ConfigAction;
//# sourceMappingURL=ConfigAction.js.map