"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EngineRunProgressSpinner = exports.RuleSelectionProgressSpinner = void 0;
const code_analyzer_core_1 = require("@salesforce/code-analyzer-core");
const messages_1 = require("../messages");
class ProgressSpinner {
    display;
    tickTime;
    tickIntervalId;
    _isSpinning;
    startTime;
    constructor(display, tickTime = 1000) {
        this.display = display;
        this.tickTime = tickTime;
        this._isSpinning = false;
    }
    startSpinning(action) {
        /* istanbul ignore if: Shouldn't be possible for spinner to start twice */
        if (this._isSpinning) {
            return;
        }
        this.startTime = Date.now();
        this.display.spinnerStart(action, this.createSpinnerStatus());
        if (this.tickTime > 0) {
            // `setInterval` means the callback will be called repeatedly. This allows us to automatically refresh the
            // spinner on a regular interval even if nothing happened. This is primarily useful for incrementing a timer,
            // so the user doesn't feel like the system is frozen.
            this.tickIntervalId = setInterval(() => {
                this.display.spinnerUpdate(this.createSpinnerStatus());
            }, this.tickTime);
        }
        this._isSpinning = true;
    }
    updateSpinner() {
        this.display.spinnerUpdate(this.createSpinnerStatus());
    }
    stopSpinning() {
        if (this.tickIntervalId) {
            clearInterval(this.tickIntervalId);
        }
        if (this._isSpinning) {
            this.display.spinnerStop((0, messages_1.getMessage)(messages_1.BundleName.ProgressEventListener, 'base-spinner.done'));
            this._isSpinning = false;
        }
    }
    isSpinning() {
        return this._isSpinning;
    }
    /**
     * Return the number of seconds (rounded down) that the spinner has been spinning.
     * @protected
     */
    getSecondsSpentSpinning() {
        return Math.floor((Date.now() - this.startTime) / 1000);
    }
}
class RuleSelectionProgressSpinner extends ProgressSpinner {
    isListening;
    engineNames;
    completionPercentages;
    /**
     * @param display The display with which to show output to the user
     * @param tickTime A number of milliseconds to wait between automatic ticks. Defaults to 1000. Negative values disable ticking.
     */
    constructor(display, tickTime = 1000) {
        super(display, tickTime);
        this.isListening = false;
        this.engineNames = new Set();
        this.completionPercentages = [];
    }
    listen(...codeAnalyzers) {
        /* istanbul ignore else: should not be calling listen() twice */
        if (!this.isListening) {
            this.isListening = true;
            codeAnalyzers.forEach((codeAnalyzer, idx) => {
                for (const engineName of codeAnalyzer.getEngineNames()) {
                    this.engineNames.add(engineName);
                }
                this.completionPercentages.push(0);
                codeAnalyzer.onEvent(code_analyzer_core_1.EventType.RuleSelectionProgressEvent, (e) => this.handleEvent(e, idx));
            });
        }
    }
    stopListening() {
        if (this.isListening) {
            this.isListening = false;
            this.stopSpinning();
        }
    }
    handleEvent(e, idx) {
        if (!this.isListening) {
            return;
        }
        if (!this.isSpinning()) {
            this.startSpinning((0, messages_1.getMessage)(messages_1.BundleName.ProgressEventListener, 'selection-spinner.action'));
        }
        this.completionPercentages[idx] = Math.floor(e.percentComplete);
        this.updateSpinner();
        // Since the events this spinner listens to have one aggregated completion percentage, we can (and should) stop
        // listening as soon as all Cores get their 100% event, instead of waiting for the `stopListening()` method.
        if (this.completionPercentages.every(p => p === 100)) {
            this.stopListening();
        }
    }
    createSpinnerStatus() {
        const aggregateCompletionPercentage = Math.floor(this.completionPercentages.reduce((prevTotal, next) => prevTotal + next, 0) / this.completionPercentages.length);
        return (0, messages_1.getMessage)(messages_1.BundleName.ProgressEventListener, 'selection-spinner.status', [[...this.engineNames.keys()].join(', '), aggregateCompletionPercentage, this.getSecondsSpentSpinning()]);
    }
}
exports.RuleSelectionProgressSpinner = RuleSelectionProgressSpinner;
class EngineRunProgressSpinner extends ProgressSpinner {
    isListening;
    /**
     * Mapping from each engine's name to a number indicating its completion percentage.
     * @private
     */
    progressMap;
    /**
     * @param display The display with which to show output to the user
     * @param tickTime A number of milliseconds to wait between automatic ticks. Defaults to 1000. Negative values disable ticking.
     */
    constructor(display, tickTime = 1000) {
        super(display, tickTime);
        this.isListening = false;
    }
    listen(...codeAnalyzers) {
        if (codeAnalyzers.length > 1) {
            // It's technically possible to implement multiple-core support for this spinner, but not currently necessary.
            // So just throw a Developer Error if anyone tries.
            throw new Error('Developer Error: EngineRunProgressSpinner cannot listen to multiple Code Analyzers');
        }
        /* istanbul ignore else: should not be calling listen() twice */
        if (!this.isListening) {
            this.isListening = true;
            this.progressMap = new Map();
            codeAnalyzers[0].onEvent(code_analyzer_core_1.EventType.EngineRunProgressEvent, (e) => this.handleEvent(e));
        }
    }
    stopListening() {
        if (this.isListening) {
            this.isListening = false;
            this.stopSpinning();
        }
    }
    handleEvent(e) {
        /* istanbul ignore if: no event should be coming in after we're done listening */
        if (!this.isListening) {
            return;
        }
        // Update the progress map at the start so that the updated information is used in the next status.
        this.progressMap.set(e.engineName, e.percentComplete);
        if (!this.isSpinning()) {
            this.startSpinning((0, messages_1.getMessage)(messages_1.BundleName.ProgressEventListener, 'execution-spinner.action'));
        }
        this.updateSpinner();
    }
    createSpinnerStatus() {
        const secondsRunning = this.getSecondsSpentSpinning();
        const totalEngines = this.progressMap.size;
        let unfinishedEngines = 0;
        const engineLines = [];
        for (const [name, progress] of this.progressMap.entries()) {
            if (progress !== 100) {
                unfinishedEngines += 1;
            }
            engineLines.push((0, messages_1.getMessage)(messages_1.BundleName.ProgressEventListener, 'execution-spinner.engine-status', [name, progress]));
        }
        return [
            (0, messages_1.getMessage)(messages_1.BundleName.ProgressEventListener, 'execution-spinner.progress-summary', [unfinishedEngines, totalEngines, secondsRunning]),
            ...engineLines
        ].join('\n');
    }
}
exports.EngineRunProgressSpinner = EngineRunProgressSpinner;
//# sourceMappingURL=ProgressEventListener.js.map